if not ClosetGnome then return end

-------------------------------------------------------------------------------
-- Localization                                                              --
-------------------------------------------------------------------------------

local al = LibStub("AceLocale-3.0")
do do local L = al:NewLocale("ClosetGnome-Banker", "enUS", true)
	L["toggle_desc"] = "Toggle whether to always keep this set in your inventory or if it should be put into the bank automatically when visited."
	L["addon_desc"] = "ClosetGnome Banker automatically gets and puts your item sets from and to your bank when you visit it based on the settings below. A checked set will always stay in your inventory, an unchecked one will be flushed to the bank with a 5 minute interval.\n\n"
	L["Banker"] = true
end do local L = al:NewLocale("ClosetGnome-Banker", "ruRU") if L then
	L["Banker"] = "Банкир"
end end do local L = al:NewLocale("ClosetGnome-Banker", "frFR") if L then
	L["Banker"] = "Banquier"
end end do local L = al:NewLocale("ClosetGnome-Banker", "zhCN") if L then
	L["Banker"] = "银行"
end end do local L = al:NewLocale("ClosetGnome-Banker", "koKR") if L then
	L["Banker"] = "은행"
end end do local L = al:NewLocale("ClosetGnome-Banker", "esES") if L then
	L["Banker"] = "Banquero"
end end do local L = al:NewLocale("ClosetGnome-Banker", "zhTW") if L then
	L["Banker"] = "銀行"
end end end

-------------------------------------------------------------------------------
-- Locals                                                                    --
-------------------------------------------------------------------------------

local L = al:GetLocale("ClosetGnome-Banker")

local bankAvailable = nil
local bankSlots = { -1, 5, 6, 7, 8, 9, 10, 11 }
local slotLocks = nil
local lastFlush = nil

-------------------------------------------------------------------------------
-- Addon declaration                                                         --
-------------------------------------------------------------------------------
local addon = CreateFrame("Frame")
addon:SetScript("OnEvent", function(self, event, ...) self[event](self, ...) end)

-------------------------------------------------------------------------------
-- Initialization                                                            --
-------------------------------------------------------------------------------

local options = {
	type = "group",
	name = "Banker",
	args = {
		bankerDescription = {
			type = "description",
			name = L["addon_desc"],
			order = 1,
		},
	},
}

local function disable()
	return not bankAvailable
end
local function toggle(info, val)
	local set = info.option.name
	addon.db.char.keep[set] = val and true or nil
	lastFlush = nil -- Flush again on next bank open
end
local function get(info)
	local set = info.option.name
	return addon.db.char.keep[set]
end

local sets = {}
local function updateSets()
	for k in pairs(ClosetGnome.db.char.set) do
		table.insert(sets, k)
	end
	table.sort(sets)
	for k, v in pairs(options.args) do
		if k ~= "bankerDescription" then
			v.hidden = true
		end
	end
	local counter = 10
	for i, set in pairs(sets) do
		if options.args[set] then
			options.args[set].hidden = nil
			options.args[set .. "_image"].hidden = nil
		else
			options.args[set] = {
				type = "toggle",
				name = set,
				desc = L["toggle_desc"],
				get = get,
				set = toggle,
				order = counter,
			}
			options.args[set .. "_image"] = {
				type = "description",
				name = "",
				image = ClosetGnome.db.char.icons[set] or "Interface\\Icons\\INV_Chest_Cloth_17",
				order = counter,
				width = "half",
			}
			options.args[set .. "_spacer"] = {
				type = "description",
				name = "",
				order = counter + 2,
			}
		end
		counter = counter + 10
	end
	for k in pairs(sets) do sets[k] = nil end
end

local function init()
	addon.db = LibStub("AceDB-3.0"):New("ClosetGnomeBankerDB", {char = {keep = {}}})

	addon:RegisterEvent("BANKFRAME_OPENED")
	addon:RegisterEvent("BANKFRAME_CLOSED")

	ClosetGnome.RegisterCallback(addon, "AddSet", updateSets)
	ClosetGnome.RegisterCallback(addon, "DeleteSet", updateSets)

	updateSets()

	LibStub("AceConfig-3.0"):RegisterOptionsTable("ClosetGnome-Banker", options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ClosetGnome-Banker", L["Banker"], "ClosetGnome")
end

ClosetGnome:RegisterCallback("Initialized", init)

-------------------------------------------------------------------------------
-- Meat                                                                      --
-------------------------------------------------------------------------------

function addon:Flush()
	if CursorHasItem() or not bankAvailable then return false end

	-- Okay, so, we have to do three things:
	-- 1. Build a list of all the items in all the sets from CG.
	-- 2. Remove the items in the Keep sets from that list.
	-- 3. Move the remaining items from the inventory to the bank.

	-- Step 1.
	local items = {}
	for k, v in pairs(ClosetGnome.db.char.set) do
		for id, item in pairs(v) do
			items[item] = true
		end
	end

	-- Step 2.
	for k in pairs(self.db.char.keep) do
		if not ClosetGnome.db.char.set[k] then
			self.db.char.keep[k] = nil
		else
			for id, item in pairs(ClosetGnome.db.char.set[k]) do
				items[item] = nil
			end
		end
	end

	-- Step 3.
	for item in pairs(items) do
		local fromBagId, fromSlotId = ClosetGnome:FindItem(item)
		if fromBagId > -1 then
			local toBagId, toSlotId = self:FindFreeBankSlot()
			if toBagId == nil then break end
			if not slotLocks then slotLocks = {} end
			if not slotLocks[toBagId] then slotLocks[toBagId] = {} end
			slotLocks[toBagId][toSlotId] = true
			PickupContainerItem(fromBagId, fromSlotId)
			PickupContainerItem(toBagId, toSlotId)
		end
	end

	slotLocks = nil
end

function addon:FlushAndGet()
	if CursorHasItem() or not bankAvailable then return false end

	self:Flush()

	for k in pairs(self.db.char.keep) do
		self:RetrieveSet(k)
	end
end

function addon:IsBankBag(bagId)
	if bagId == -1 then return true end
	if bagId > 11 or (bagId < 5 and bagId ~= -1) then return false end
	local link = GetInventoryItemLink("player", ContainerIDToInventoryID(bagId))
	if not link then return false end
	local linkId = select(3, link:find("item:(%d+)"))
	if not linkId then return false end
	local bagType = select(7, GetItemInfo(linkId))
	if bagType and bagType == INVTYPE_BAG then return true end
	return false
end

function addon:FindFreeBankSlot()
	if not bankAvailable then return nil end
	for i, v in ipairs(bankSlots) do
		if self:IsBankBag(v) then
			if not slotLocks then slotLocks = {} end
			if not slotLocks[v] then slotLocks[v] = {} end
			local numSlots = GetContainerNumSlots(v)
			for slot = 1, numSlots do
				if not slotLocks[v][slot] then
					local tex = GetContainerItemInfo(v, slot)
					if not tex then
						return v, slot
					end
				end
			end
		end
	end
	return nil
end

function addon:FindFreeBagSlot()
	for theBag = NUM_BAG_FRAMES, 0, -1 do
		if ClosetGnome:IsNormalBag(theBag) then
			if not slotLocks then slotLocks = {} end
			if not slotLocks[theBag] then slotLocks[theBag] = {} end
			local numSlot = GetContainerNumSlots(theBag)
			for theSlot = 1, numSlot do
				if not slotLocks[theBag][theSlot] then
					local texture = GetContainerItemInfo(theBag, theSlot)
					if not texture then
						return theBag, theSlot
					end
				end
			end
		end
	end
	return nil
end

function addon:StoreSet(setName)
	if CursorHasItem() or not setName or type(setName) ~= "string" or not bankAvailable then return false end
	local set = ClosetGnome.db.char.set[setName]
	if not set then return false end

	for _, item in pairs(set) do
		local fromBagId, fromSlotId = ClosetGnome:FindItem(item)
		if fromBagId > -1 then
			local toBagId, toSlotId = self:FindFreeBankSlot()
			if toBagId == nil then break end
			if not slotLocks then slotLocks = {} end
			if not slotLocks[toBagId] then slotLocks[toBagId] = {} end
			slotLocks[toBagId][toSlotId] = true
			PickupContainerItem(fromBagId, fromSlotId)
			PickupContainerItem(toBagId, toSlotId)
		end
	end

	slotLocks = nil
end

function addon:RetrieveSet(setName)
	if CursorHasItem() or not setName or type(setName) ~= "string" or not bankAvailable then return false end
	local set = ClosetGnome.db.char.set[setName]
	if not set then return false end

	for i, fromBagId in ipairs(bankSlots) do
		if self:IsBankBag(fromBagId) then
			local numSlots = GetContainerNumSlots(fromBagId)
			if not slotLocks then slotLocks = {} end
			if not slotLocks[fromBagId] then slotLocks[fromBagId] = {} end
			for fromSlotId = 1, numSlots do
				local itemLink = GetContainerItemLink(fromBagId, fromSlotId)
				if itemLink then
					for _, item in pairs(set) do
						if itemLink == item and not slotLocks[fromBagId][fromSlotId] then
							-- Found the item in the bank.
							local toBagId, toSlotId = self:FindFreeBagSlot()
							if toBagId ~= nil then
								slotLocks[toBagId][toSlotId] = true
								slotLocks[fromBagId][fromSlotId] = true
								PickupContainerItem(fromBagId, fromSlotId)
								PickupContainerItem(toBagId, toSlotId)
							end
						end
					end
				end
			end
		end
	end

	slotLocks = nil
end

-- Checks whether the given set is either in your inventory or is already equipped.
function addon:IsSetInInventory(name)
	local set = ClosetGnome.db.char.set[name]
	if not set then return false end
	local avail = true
	for _, item in pairs(set) do
		local bag = ClosetGnome:FindItem(item)
		avail = avail and ((bag > -1) or IsEquippedItem(item))
	end
	return avail
end

-------------------------------------------------------------------------------
-- Events                                                                    --
-------------------------------------------------------------------------------

function addon:BANKFRAME_OPENED()
	bankAvailable = true
	if not lastFlush or GetTime() > (lastFlush + 600) then
		-- More than 5 minutes since last flush, so do it again.
		lastFlush = GetTime()
		self:FlushAndGet()
	end
end

function addon:BANKFRAME_CLOSED()
	bankAvailable = nil
end

--------------------------------------------------------------------------------
-- Hooks
--------------------------------------------------------------------------------

do
	local orig = ClosetGnome.WearSet
	ClosetGnome.WearSet = function(self, ...)
		if bankAvailable then
			local set = select(1, ...)
			if not addon:IsSetInInventory(set) then
				addon:RetrieveSet(set)
				return false
			end
		end
		return orig(self, ...)
	end
end

